@@ -1,10 +1,16 @@
 from django.conf import settings
 from django.contrib.sites.models import Site
+from django.contrib import messages
 from django.urls import reverse
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from ryzom_django_mdc.html import *  # noqa
 
 
+class Html(Html):
+    def to_html(self, *args, **kwargs):
+        return '<!doctype html>' + super().to_html(*args, **kwargs)
+
+
 UNPOLY_TARGET_ALL = '#main, .mdc-top-app-bar__title, #drawer .mdc-list'
 
 
@@ -54,44 +60,71 @@ class FormContainer(Container):
 class PageMenu(Div):
     attrs = dict(cls='mdc-elevation--z2', style='margin-bottom: 10px')
 
-    def __init__(self, *args, _next=None, **kwargs):
+    def __init__(self, *args, _next=None, _next_destructible=False, **kwargs):
         super().__init__(*args, **kwargs)
         self._next = _next
+        # wether the next page is destructible, such as a detail view after a
+        # delete post
+        self._next_destructible = _next_destructible
+
+    def link(self, view, context):
+        button = A(
+            MDCTextButton(
+                view.label.capitalize(),
+                icon=getattr(view, 'icon', None),
+                tag='span',
+                style={
+                    'margin': '10px',
+                    'color': getattr(view, 'color', 'inherit'),
+                },
+            ),
+            href=view.url,
+            style='text-decoration: none',
+        )
+
+        if getattr(view, 'controller', None) != 'modal':
+            return button
+
+        # wether going from the here to the next url is safe,
+        # safe means that this menu view will not destroy the current url
+        # which will be the case if we're in a detail view and making a
+        # DELETE request on the object
+        dangerous = (
+            self._next_destructible
+            and getattr(view, 'destructive', False)
+        )
+
+        button.attrs.up_layer = 'new'
+        if dangerous:
+            # in case the view will destroy the current view, enforce
+            # list view
+            next_url = str(view.router['list'].url)
+            button.attrs.up_on_accepted = (
+                f'up.visit("{view.router["list"].url}")'
+            )
+        else:
+            next_url = self._next
+            # we have to reload meanwhile unpoly 2 learns to reuse the
+            # form response
+            button.attrs.up_on_accepted = 'up.reload()'
+
+        if '?' not in button.attrs['href']:
+            button.attrs['href'] += '?'
+        button.attrs['href'] += '&_next=' + next_url
+        button.attrs.up_accept_location = next_url
+        del button.attrs['up-target']
+        return button
 
     def to_html(self, *content, **context):
         if 'page-menu' not in context:
             return super().to_html(*content, **context)
 
         content = list(content)
-        menu = context['page-menu']
-
-        for v in menu:
-            if v.urlname == context['view'].urlname:
-                continue
-
-            href = v.url
-            if self._next:
-                if '?' not in href:
-                    href += '?'
-                href += '&_next=' + self._next
-            button = A(
-                MDCTextButton(
-                    v.label.capitalize(),
-                    icon=getattr(v, 'icon', None),
-                    tag='span',
-                    style={
-                        'margin': '10px',
-                        'color': getattr(v, 'color', 'inherit'),
-                    },
-                ),
-                href=href,
-                style='text-decoration: none',
-            )
-            if getattr(v, 'controller', None) == 'modal':
-                button.attrs.up_modal = '.main-inner'
-                del button.attrs['up-target']
-
-            content.append(button)
+        content += [
+            self.link(view, context)
+            for view in context['page-menu']
+            if view.urlname != context['view'].urlname
+        ]
 
         return super().to_html(
             *content,
@@ -105,47 +138,72 @@ class Main(Main):
     pass
 
 
-class ModalClose(Component):
+class Message(MDCSnackBar):
+    icons = dict(
+        debug='check_circle',
+        info='check_circle',
+        success='check_circle',
+        warning='warning',
+        error='error',
+    )
+    colors = dict(
+        debug='green',
+        info='yellow',
+        success='green',
+        warning='orange',
+        error='red',
+    )
+    style = '''
+        padding-left: 16px;
+        display: flex;
+        flex-direction: row;
+        align-items: center
     '''
-    Close the modal when inserted by Unpoly.
 
-    This should be the default behaviour of Unpoly, as documented, and is the
-    case with modals that do not redirect on the same page, but for modals with
-    a form with a ?_next= redirection then surprisingly unpoly does update the
-    body but does not close the modal by default.
 
-    This tag registers a compiler that calls up.modal.close() when it
-    sees itself, given that unpoly does not execute scripts in
-    response that it loads. As such, this tag MUST be in the initial body, but
-    outside the elements that you will load in modal.
+class Messages(Div):
+    def to_html(self, *content, **context):
+        msgs = messages.get_messages(context['view'].request)
+        if not msgs:
+            return ''
 
-    Remove this hack when we figure the problem in unpoly.
-
-    Equivalent of::
-
-        Script(
-            'up.compiler(".closemodal", function() { up.modal.close(); })',
-            cls='closemodal',
-        ),
-    '''
-    attrs = dict(cls='closemodal')
-
-    def py2js():
-        up.compiler('.closemodal', lambda: up.modal.close())
+        return Div(*[
+            Message(message)
+            for message in msgs
+        ]).to_html(*content, **context)
 
 
 class Body(Body):
     style = 'margin: 0'
+    sass = '''
+    up-progress-bar
+        background-color: var(--mdc-theme-secondary)
+        height: 24px
+        bottom: 0
+        top: auto !important
+        background: linear-gradient( \
+            90deg, \
+            var(--mdc-theme-primary), \
+            var(--mdc-theme-secondary) \
+        )
+        width: 200vw
+        animation: rotate 1s linear infinite
+        position: absolute
+
+    up-modal-box
+        overflow: initial !important
+    '''
 
     def __init__(self, *content, **attrs):
         self.drawer = mdcDrawer(id='drawer')
         self.bar = mdcTopAppBar()
+        self.main_inner = Div(
+            Messages(),
+            *content,
+            cls='main-inner',
+        )
         self.main = Main(
-            Div(
-                *content,
-                cls='main-inner',
-            ),
-            ModalClose(),
+            self.main_inner,
             cls='main mdc-drawer-app-content',
             id='main',
         )
@@ -168,101 +226,63 @@ class Body(Body):
             up.log.enable()
 
 
-class SpinnerOverlay(Div):
+class Spinner(Div):
     sass = '''
-    .SpinnerOverlay
+    .Spinner
         position: fixed
-        width: 100%
-        height: 100%
-        top: 0
-        left: 0
-        right: 0
-        bottom: 0
-        background-color: rgba(255, 255, 255, 0.66)
+        float: right
         z-index: 20000
         cursor: pointer
-        padding-top: 30vh
 
-        .loader
-            color: var(--mdc-theme-primary)
-            opacity: 50%
-            font-size: 90px
-            text-indent: -9999em
-            overflow: hidden
-            width: 1em
-            height: 1em
-            border-radius: 50%
-            margin: 72px auto
-            position: relative
-            -webkit-transform: translateZ(0)
-            -ms-transform: translateZ(0)
-            transform: translateZ(0)
-            -webkit-animation: load6 1.7s infinite ease, round 1.7s infinite ease
-            animation: load6 1.7s infinite ease, round 1.7s infinite ease
+        right: -24px
+        top: 3px
+        @media only screen and (max-width: 600px)
+          top: -32px
+          top: 0
 
-        @-webkit-keyframes load6
-            0%
-                box-shadow: 0 -0.83em 0 -0.4em, 0 -0.83em 0 -0.42em, 0 -0.83em 0 -0.44em, 0 -0.83em 0 -0.46em, 0 -0.83em 0 -0.477em
-            5%, 95%
-                box-shadow: 0 -0.83em 0 -0.4em, 0 -0.83em 0 -0.42em, 0 -0.83em 0 -0.44em, 0 -0.83em 0 -0.46em, 0 -0.83em 0 -0.477em
-            10%, 59%
-                box-shadow: 0 -0.83em 0 -0.4em, -0.087em -0.825em 0 -0.42em, -0.173em -0.812em 0 -0.44em, -0.256em -0.789em 0 -0.46em, -0.297em -0.775em 0 -0.477em
-            20%
-                box-shadow: 0 -0.83em 0 -0.4em, -0.338em -0.758em 0 -0.42em, -0.555em -0.617em 0 -0.44em, -0.671em -0.488em 0 -0.46em, -0.749em -0.34em 0 -0.477em
-            38%
-                box-shadow: 0 -0.83em 0 -0.4em, -0.377em -0.74em 0 -0.42em, -0.645em -0.522em 0 -0.44em, -0.775em -0.297em 0 -0.46em, -0.82em -0.09em 0 -0.477em
-            100%
-                box-shadow: 0 -0.83em 0 -0.4em, 0 -0.83em 0 -0.42em, 0 -0.83em 0 -0.44em, 0 -0.83em 0 -0.46em, 0 -0.83em 0 -0.477em
-        @keyframes load6
-            0%
-                box-shadow: 0 -0.83em 0 -0.4em, 0 -0.83em 0 -0.42em, 0 -0.83em 0 -0.44em, 0 -0.83em 0 -0.46em, 0 -0.83em 0 -0.477em
-            5%, 95%
-                box-shadow: 0 -0.83em 0 -0.4em, 0 -0.83em 0 -0.42em, 0 -0.83em 0 -0.44em, 0 -0.83em 0 -0.46em, 0 -0.83em 0 -0.477em
-            10%, 59%
-                box-shadow: 0 -0.83em 0 -0.4em, -0.087em -0.825em 0 -0.42em, -0.173em -0.812em 0 -0.44em, -0.256em -0.789em 0 -0.46em, -0.297em -0.775em 0 -0.477em
-            20%
-                box-shadow: 0 -0.83em 0 -0.4em, -0.338em -0.758em 0 -0.42em, -0.555em -0.617em 0 -0.44em, -0.671em -0.488em 0 -0.46em, -0.749em -0.34em 0 -0.477em
-            38%
-                box-shadow: 0 -0.83em 0 -0.4em, -0.377em -0.74em 0 -0.42em, -0.645em -0.522em 0 -0.44em, -0.775em -0.297em 0 -0.46em, -0.82em -0.09em 0 -0.477em
-            100%
-                box-shadow: 0 -0.83em 0 -0.4em, 0 -0.83em 0 -0.42em, 0 -0.83em 0 -0.44em, 0 -0.83em 0 -0.46em, 0 -0.83em 0 -0.477em
-        @-webkit-keyframes round
-            0%
-                -webkit-transform: rotate(0deg)
-                transform: rotate(0deg)
-            100%
-                -webkit-transform: rotate(360deg)
-                transform: rotate(360deg)
-        @keyframes round
-            0%
-                -webkit-transform: rotate(0deg)
-                transform: rotate(0deg)
-            100%
-                -webkit-transform: rotate(360deg)
-                transform: rotate(360deg)
+        .lds-dual-ring
+          display: inline-block
+          width: 80px
+          height: 80px
+
+        .lds-dual-ring:after
+          content: " "
+          display: block
+          width: 28px
+          height: 28px
+          margin: 8px
+          border-radius: 50%
+          border: 6px solid #fff
+          border-color: var(--mdc-theme-primary) white var(--mdc-theme-primary) white
+          animation: lds-dual-ring 1.2s linear infinite
+
+        @keyframes lds-dual-ring
+          0%
+            transform: rotate(0deg)
+
+          100%
+            transform: rotate(360deg)
     '''  # noqa
 
     def py2js(self):
         up.proxy.config.slowDelay = 25
-        up.compiler('.SpinnerOverlay', lambda element: [
-            up.on('up:proxy:slow', lambda: up.element.show(element)),
-            up.on('up:proxy:recover', lambda: up.element.hide(element)),
+        up.compiler('.Spinner', lambda element: [
+            up.on('up:request:late', lambda: up.element.show(element)),
+            up.on('up:request:recover', lambda: up.element.hide(element)),
         ])
 
     def __init__(self):
-        super().__init__(Div('...', cls='loader'), style='display:none')
+        super().__init__(Div(cls='lds-dual-ring'), style='display:none')
 
 
 class App(Html):
     body_class = Body
     scripts = [
-        'https://unpkg.com/unpoly@1.0.0/dist/unpoly.js',
-        # 'https://unpkg.com/unpoly@2.0.0-rc9/unpoly.min.js',
-        # 'https://unpkg.com/unpoly@2.0.0-rc9/unpoly-migrate.js',
+        'https://unpkg.com/unpoly@2.6.1/unpoly.js',
+        'https://unpkg.com/unpoly@2.6.1/unpoly-migrate.js',
     ]
     stylesheets = [
-        'https://unpkg.com/unpoly@1.0.0/dist/unpoly.css',
-        # 'https://unpkg.com/unpoly@2.0.0-rc9/unpoly.min.css',
+        'https://unpkg.com/unpoly@2.6.1/unpoly.min.css',
     ]
     sass = '''
     .up-modal .up-modal-viewport
@@ -274,7 +294,7 @@ class App(Html):
     def to_html(self, *content, **context):
         if title := getattr(context['view'], 'title', None):
             self.head.content.append(Title(title))
-        return super().to_html(*content, SpinnerOverlay(), **context)
+        return super().to_html(*content, **context)
 
 
 class NarrowCard(Div):
@@ -306,13 +326,24 @@ class FormTemplate(FormContainer):
                 ),
                 href=next_,
             )
+
+        _next = view.request.GET.get(
+            '_next',
+            view.request.POST.get('_next', ''),
+        )
+        if _next:
+            _next = Input(type='hidden', name='_next', value=_next)
+
         return super().to_html(
             H3(view.title),
             Form(
                 form,
                 CSRFInput(view.request),
+                _next,
                 back,
                 MDCButtonRaised(getattr(view, 'title_submit', _('Submit'))),
+                method='post',
+                action=view.request.path_info,
             ),
         )
 
@@ -354,14 +385,20 @@ class Swagger(Div):
 class Home(Div):
     def to_html(self, **context):
         site = Site.objects.get_current()
+        user = context['view'].request.user
         return super().to_html(
-            H1('Welcome to ' + site.name),
+            H1(
+                f'Welcome to {site.name} {user.email}'
+                if user.is_authenticated
+                else
+                f'Welcome to {site.name}!'
+            ),
             MDCButtonRaised(
                 'Login to continue',
                 href=reverse('login'),
                 up_target=UNPOLY_TARGET_ALL,
                 tag='a',
-            ),
+            ) if not user.is_authenticated else '',
             Div('Then, navigate with the menu button at the north west'),
             **context
         )
@@ -396,7 +433,10 @@ class ObjectDetail(Div):
             ))
 
         return super().to_html(
-            PageMenu(),
+            PageMenu(
+                _next=context['view'].request.path,
+                _next_destructible=True,
+            ),
             *content,
             NarrowCard(table),
             **context
@@ -446,10 +486,11 @@ class ListActions(Component):
 @template('crudlfap/list.html', App)
 class ObjectList(Div):
     def context(self, *content, **context):
-        context['page-menu'] = context['view'].router.get_menu(
-            'model',
-            context['view'].request,
-        )
+        if 'page-menu' not in context:
+            context['page-menu'] = context['view'].router.get_menu(
+                'model',
+                context['view'].request,
+            )
         return super().context(*content, **context)
 
     def to_html(self, *content, **context):
@@ -493,7 +534,10 @@ class ObjectList(Div):
         return super().to_html(
             *content,
             self.drawer_component(**context) or '',
-            PageMenu(_next=context['view'].request.path),
+            PageMenu(
+                _next=context['view'].request.path,
+                _next_destructible=False,
+            ),
             Div(
                 self.search_component(**context) or '',
                 table,
@@ -537,8 +581,8 @@ class ObjectList(Div):
         )
 
         filters_chips = Div(
-            toggle=toggle if context['view'].filterset.form.fields else '',
             search=search_form or '',
+            toggle=toggle if context['view'].filterset.form.fields else '',
             chips=Div(
                 cls='mdc-chip-set',
                 role='grid',
