@@ -16,7 +16,6 @@ class CreateMixin:
     default_template_name = 'crudlfap/create.html'
     template_name_suffixes = ['create', 'form']
     controller = 'modal'
-    action = 'click->modal#open'
     color = 'green'
     log_action_flag = ADDITION
     menus = ['model']
@@ -66,6 +65,7 @@ class DeleteMixin(ActionMixin):
     action = 'click->modal#open'
     form_class = forms.Form
     permission_shortcode = 'delete'
+    destructive = True
 
     def form_valid(self):
         if hasattr(self, 'object_list'):
@@ -75,7 +75,13 @@ class DeleteMixin(ActionMixin):
         return super().form_valid()
 
     def get_success_url(self):
-        return self.request.GET.get('_next', self.router['list'].reverse())
+        return self.request.GET.get(
+            '_next',
+            self.request.POST.get(
+                '_next',
+                self.router['list'].reverse()
+            ),
+        )
 
 
 class DetailMixin:
